package gitpb

import (
	"errors"
	"fmt"
	"os"
	"path/filepath"
	"strings"

	"go.wit.com/log"
)

// returns true if 'git pull' will work
func (repo *Repo) ExistsUserBranchRemote() bool {
	branchname := repo.GetUserBranchName()
	if repo.IsBranchRemote(branchname) {
		return true
	}
	return false
}

func readRefHash(filename string) string {
	data, _ := os.ReadFile(filename)
	return string(data)
}

func (repo *Repo) GetLocalBranches() []string {
	return ListFiles(filepath.Join(repo.GetFullPath(), "/.git/refs/heads"))
}

func (repo *Repo) GetRemoteBranches() []string {
	remotes := ListFiles(filepath.Join(repo.GetFullPath(), "/.git/refs/remotes"))
	return remotes
}

// git describe --tags e548b0fb6d0d14cdfb693850d592419f247dc2b1
// v0.22.61-15-gbab84d7
func (repo *Repo) GetHashName(h string) (string, error) {
	h = strings.TrimSpace(h)
	// log.Info("GetHashName() is looking for", repo.GetGoPath(), h)
	cmd := []string{"git", "describe", "--tags", h}
	r, err := repo.RunQuiet(cmd)
	if err != nil {
		return "", err
	}
	if len(r.Stdout) == 0 {
		return "", errors.New("git describe was empty")
	}
	return r.Stdout[0], nil
}

// lookup a hash from a tag with 'git rev-list'
func (repo *Repo) GetTagHash(t string) string {
	// git rev-list -n 1 v0.0.66
	cmd := []string{"git", "rev-list", "-n", "1", t}
	result, _ := repo.RunStrict(cmd)
	// log.Info("getLastTagVersion()", result.Stdout)

	if len(result.Stdout) == 0 {
		// log.Log(WARN, "no gitpb.LastTag() repo is broken. ignore this.", repo.GetGoPath())
		return ""
	}

	return result.Stdout[0]
}

// deletes the devel local branch if it is a subset of the remote devel branch
func (repo *Repo) DeleteLocalDevelBranch() error {
	branch := repo.GetDevelBranchName()
	remote := filepath.Join("origin", branch)

	if !repo.IsDevelRemote() {
		return fmt.Errorf("no remote branch")
	}
	b1 := repo.CountDiffObjects(branch, remote) // should be zero
	if b1 == 0 {
		cmd := []string{"git", "branch", "-D", repo.GetDevelBranchName()}
		log.Info("DEVEL IS IN REMOTE", repo.GetGoPath(), cmd)
		err := repo.RunVerbose(cmd)
		return err
	} else {
		return fmt.Errorf("local branch has patches not in remote")
	}
}
