package gitpb

import (
	"errors"
	"fmt"

	"go.wit.com/log"
)

func (repo *Repo) isTracked(file string) (bool, error) {
	cmd := []string{"git", "ls-files", "--error-unmatch", file}
	result := repo.Run(cmd)
	if result.Error != nil {
		return false, result.Error
	}
	if result.Exit != 0 {
		return false, nil
	}
	return true, nil
}

func (repo *Repo) isIgnored(file string) (bool, error) {
	cmd := []string{"git", "check-ignore", "-q", file}
	result := repo.Run(cmd)
	if result.Error != nil {
		return false, result.Error
	}
	if result.Exit == 0 {
		// exit with 0 means the file is ignored
		return true, nil
	}
	// non-zero exit means the file is not ignored
	return false, nil
}

// is it a good idea to run go-mod-clean in this repo?
// for now, check if this repo should be ignored
// TODO: go.mod and go.sum should be moved to git tag metadata
func (repo *Repo) RepoIgnoresGoMod() error {
	repo.GoInfo.GitIgnoresGoSum = false
	file := "go.mod"
	if tracked, err := repo.isTracked(file); err != nil {
		msg := fmt.Sprintf("%s Error checking if %s tracked: %v\n", repo.GetGoPath(), file, err)
		log.Info("gitpb:", msg)
		return err
	} else {
		if tracked {
			msg := fmt.Sprintf("%s %s is tracked by Git.\n", repo.GetGoPath(), file)
			log.Info("gitpb:", msg)
			return errors.New(msg)
		}
	}

	if ignored, err := repo.isIgnored(file); err != nil {
		if err != nil {
			msg := fmt.Sprintf("%s Error checking if ignored: %v\n", repo.GetGoPath(), err)
			log.Info("gitpb:", msg)
			return err
		}
	} else {
		if ignored {
			fmt.Printf("%s %s is ignored by Git.\n", repo.GetGoPath(), file)
			repo.GoInfo.GitIgnoresGoSum = true
			return nil
		}
	}
	msg := fmt.Sprintf("%s %s is neither tracked nor ignored by Git.\n", repo.GetGoPath(), file)
	// this means, if you make a go.mod file, it'll add it to the repo to be tracked
	// so you need to either add it to .gitignore (this is what should happen)
	// or accept you want an auto-generated file to put endless garbage in your git repo
	// this obviously exposes my opinion on this subject matter
	log.Info("gitpb:", msg)
	return errors.New(msg)
}
